﻿using AlienRace;
using RimWorld;
using System;
using System.Collections.Generic;
using Verse;

namespace ChangeLook
{
    //메소드
    public class ChangeLookDef : Def
    {
        public ThingDef_AlienRace targetRace = new ThingDef_AlienRace();
        public DefaultHediff defaultHediff = new DefaultHediff();
        public PrefixAndPostfix prefixAndPostfix = new PrefixAndPostfix();
        public List<AgeProperty> ageProperty = new List<AgeProperty>();
        public Property property = new Property();
        public List<UnDisplay> unDisplay = new List<UnDisplay>();
        public List<BodyTypeDef> bodyTypeDefList = new List<BodyTypeDef>();

        public void StartChangeLook(Pawn pawn, string becauseOf)
        {
            ThingComp_ChangeLook thingComp_ChangeWithAge = pawn.TryGetComp<ThingComp_ChangeLook>();
            thingComp_ChangeWithAge.bodycheck = true;
            thingComp_ChangeWithAge.becauseOf = becauseOf;
        }

        //폰이 존재하는지는 알아서
        public void Changelook(Pawn input)
        {
            if (input.def == targetRace && ChangeLook_Setting.OnOffChangeLook)
            {
                TestErrorLog("============== ChangelooktestError Start ==============");
                TestErrorLog($"ChangeLookDef DefName : {defName}");
                TestErrorLog($"TargetRace : {targetRace}");
                TestErrorLog($"Pawn name : {input.Name}");
                TestErrorLog($"Pawn race : {input.def.defName}");
                TestErrorLog($"Pawn age : {input.ageTracker.AgeBiologicalYears}");

                List<Hediff> hediffs = input.health.hediffSet.hediffs;
                string prefixOfBody = prefixAndPostfix.prefixOfBody;

                TestErrorLog("1!");

                if (prefixAndPostfix.prefixOfBody.NullOrEmpty() && bodyTypeDefList.Any())
                {
                    bool dontStart = true;
                    foreach (var bodyTypeDef in bodyTypeDefList)
                    {
                        if (input.story.bodyType.defName.Contains(bodyTypeDef.defName))
                        {
                            prefixOfBody = bodyTypeDef.defName;
                            dontStart = false;
                            break;
                        }
                    }
                    if (dontStart)
                    {
                        TestErrorLog($"Pawn bodyTypeDef is not satisfied with any bodyTypeDefList, it can't start Changelook.");
                        return;
                    }
                }

                if (defaultHediff.defaultElementPropertyHediff.hediffDefName == null)
                {
                    TestErrorLog("Non defaultHediff");
                }
                else
                {
                    //요소용 헤디프 리스트에 디폴트 요소용 헤디프 추가, 폰이 요소용 헤디프 리스트에 일치하는 헤디프가 없으면 폰에 디폴트 헤디프를 추가, 테스트용 로그용 스트링 출력.
                    DefaultHediffLogic(input, hediffs, defaultHediff.defaultElementPropertyHediff);
                    //바디용 헤디프 리스트에 디폴트 바디용 헤디프 추가. 폰이 바디용 헤디프 리스트에 일치하는 헤디프가 없으면 폰에 디폴트 헤디프를 추가, 테스트용 로그용 스트링 출력.
                    DefaultHediffLogic(input, hediffs, defaultHediff.defaultBodyTypePropertyHediff);
                    //테스트용 로그 출력
                }

                //폰의 헤디프와 property 대조 후 임시 PropertyName 출력
                string elementPropertyName = "";
                if (property.element.Any())
                {
                    PropertyLogic(hediffs, property.element, out elementPropertyName);
                }
                string bodyTypePropertyName = "";
                if (property.bodyType.Any())
                {
                    PropertyLogic(hediffs, property.bodyType, out bodyTypePropertyName);
                }
                string extraPropertyName = "";
                if (property.extra.Any())
                {
                    PropertyLogic(hediffs, property.extra, out extraPropertyName);
                }

                //테스트용 로그 출력
                TestErrorLog("2!", elementPropertyName, bodyTypePropertyName, extraPropertyName);


                //나이 로직
                for (int i = 0; i < ageProperty.Count; i++)
                {
                    int maxAge = ageProperty[i + 1] != null ? ageProperty[i + 1].min - 1 : 99999;
                    TestErrorLog("Start Age Logic", $"List : {i}", $"min age : {ageProperty[i].min}", $"max age : {maxAge}");

                    if (ageProperty[i].min <= input.ageTracker.AgeBiologicalYears && input.ageTracker.AgeBiologicalYears <= maxAge)
                    {
                        //나이에 따라 요소들을 변환. 만약 나이 설정에서 안쓰는 요소라면 별도의 지정된 요소로 변환해서 출력.
                        FinalPropertyNameLogic(ageProperty[i].unUsedProperty?.element, elementPropertyName, out string finalElementPropertyName);
                        FinalPropertyNameLogic(ageProperty[i].unUsedProperty?.bodyType, bodyTypePropertyName, out string finalBodyTypePropertyName);
                        FinalPropertyNameLogic(ageProperty[i].unUsedProperty?.extra, extraPropertyName, out string finalExtraPropertyName);
                        string finalAgePropertyName = ageProperty[i].ageTypeProperty;

                        //테스트용 로그 출력
                        TestErrorLog("3!", finalElementPropertyName, finalBodyTypePropertyName, finalExtraPropertyName, finalAgePropertyName);

                        //헤드 및 헤어 번호 출력
                        string finalHeadNum = "";
                        if (ageProperty[i].headElementNumber != null)
                        {
                            FinalHairHeadNumLogic(hediffs, ageProperty[i].headElementNumber, finalElementPropertyName, out finalHeadNum);
                        }
                        string finalHairNum = "";
                        if (ageProperty[i].hairElementNumber != null)
                        {
                            FinalHairHeadNumLogic(hediffs, ageProperty[i].hairElementNumber, finalElementPropertyName, out finalHairNum);
                        }
                        //테스트용 로그 출력
                        TestErrorLog(finalHairNum, finalHeadNum);

                        //외형 설정
                        var AgeElement = finalAgePropertyName + finalElementPropertyName;

                        //테스트용 로그 출력
                        TestErrorLog(AgeElement);

                        var FinalHair = $"{prefixAndPostfix.prefixOfHair}{AgeElement}{finalHairNum}";
                        var FinalHead = $"{prefixAndPostfix.prefixOfHead}{AgeElement}{finalHeadNum}";
                        var FinalBody = $"{prefixOfBody}{AgeElement}{finalBodyTypePropertyName}{finalExtraPropertyName}";

                        TestErrorLog("4!");

                        TestErrorLog("Original apparance : ");
                        TestErrorLog("Pawn Hair : ", input.story.hairDef?.defName ?? "null");
                        TestErrorLog("Pawn Crown : ", input.GetComp<AlienPartGenerator.AlienComp>().crownType ?? "null");
                        TestErrorLog("Pawn BodyType : ", input.story.bodyType?.defName ?? "null");

                        TestErrorLog("Try change to : ");
                        TestErrorLog("Pawn Hair : ", FinalHair);
                        TestErrorLog("Pawn Crown : ", FinalHead);
                        TestErrorLog("Pawn BodyType : ", FinalBody);


                        TestErrorLog("Strat change : ");
                        //헤어 검사

                        if (FinalHair != "")
                        {
                            if (input.story.hairDef == null)
                            {
                                input.story.hairDef = new HairDef();
                            }
                            if (DefDatabase<HairDef>.GetNamed(FinalHair) == null)
                            {
                                Log.Error($"{FinalHair} is null Def. Skip.");
                                //22.08.17 추가 
                                if (!finalExtraPropertyName.NullOrEmpty())
                                {
                                    string FinalHair_NonExtra = FinalHair.Replace(finalExtraPropertyName, "");

                                    if (DefDatabase<HairDef>.GetNamed(FinalHair_NonExtra) != null)
                                    {
                                        if (input.story.hairDef != DefDatabase<HairDef>.GetNamed(FinalHair_NonExtra) || input.story.hairDef == null)
                                        {
                                            input.story.hairDef = DefDatabase<HairDef>.GetNamed(FinalHair_NonExtra);
                                        }
                                        else
                                        {
                                            Log.Error($"{FinalBody.Replace(finalExtraPropertyName, "")} is null Def too. Skip.");
                                        }
                                    }
                                }
                            }
                            else if (!input.story.hairDef.defName.Contains(prefixAndPostfix.prefixOfHair + AgeElement) || input.story.hairDef == null || ChangeLook_Setting.ChangeHair)
                            {
                                input.story.hairDef = DefDatabase<HairDef>.GetNamed(FinalHair) ?? input.story.hairDef;
                            }
                        }
                        //크라운 검사
                        if (FinalHead != "" && (!input.GetComp<AlienPartGenerator.AlienComp>().crownType.Contains(prefixAndPostfix.prefixOfHead + AgeElement) || input.GetComp<AlienPartGenerator.AlienComp>().crownType == null || ChangeLook_Setting.ChangeHead))
                        {
                            input.GetComp<AlienPartGenerator.AlienComp>().crownType = FinalHead;
                        }
                        //바디 검사
                        //22.08.17 추가 
                        if (FinalBody != "" && DefDatabase<BodyTypeDef>.GetNamed(FinalBody) == null)
                        {
                            Log.Error(FinalBody + " is null Def. Skip.");
                            if (!finalExtraPropertyName.NullOrEmpty())
                            {
                                string FinalBody_NonExtra = FinalBody.Replace(finalExtraPropertyName, "");
                                if (DefDatabase<BodyTypeDef>.GetNamed(FinalBody_NonExtra) != null)
                                {
                                    if (input.story.bodyType != DefDatabase<BodyTypeDef>.GetNamed(FinalBody_NonExtra) || input.story.bodyType == null)
                                    {
                                        input.story.bodyType = DefDatabase<BodyTypeDef>.GetNamed(FinalBody_NonExtra);
                                    }
                                    else
                                    {
                                        Log.Error($"{FinalBody.Replace(finalExtraPropertyName, "")} is null Def too. Skip.");
                                    }
                                }
                            }
                        }
                        else if (input.story.bodyType != DefDatabase<BodyTypeDef>.GetNamed(FinalBody) || input.story.bodyType == null)
                        {
                            input.story.bodyType = DefDatabase<BodyTypeDef>.GetNamed(FinalBody);
                        }
                        //테스트용 로그 출력
                        TestErrorLog("Result : ");
                        TestErrorLog(input.story.hairDef.defName, input.GetComp<AlienPartGenerator.AlienComp>().crownType, input.story.bodyType.defName);

                        //최종 헤디프 결정
                        if (prefixAndPostfix.prefixOfHediff != "")
                        {
                            var AgeHediff = DefDatabase<HediffDef>.GetNamed(prefixAndPostfix.prefixOfHediff + AgeElement + prefixAndPostfix.postfixOfHediff);
                            var HeadHediff = DefDatabase<HediffDef>.GetNamed(input.GetComp<AlienPartGenerator.AlienComp>().crownType + prefixAndPostfix.postfixOfHediff);
                            var HairHediff = DefDatabase<HediffDef>.GetNamed(input.story.hairDef.defName + prefixAndPostfix.postfixOfHediff);

                            if (AgeHediff != null)
                            {
                                FinalHediffLogic(input, hediffs, prefixAndPostfix.prefixOfHediff, AgeElement, AgeHediff);
                            }
                            if (HeadHediff != null)
                            {
                                FinalHediffLogic(input, hediffs, prefixAndPostfix.prefixOfHead, AgeElement, HeadHediff);
                            }
                            if (HairHediff != null)
                            {
                                FinalHediffLogic(input, hediffs, prefixAndPostfix.prefixOfHair, AgeElement, HairHediff);
                            }

                            //테스트용 로그 출력
                            TestErrorLog("5!");
                            TestErrorLog(HairHediff?.defName ?? "null hair hediff def", HeadHediff?.defName ?? "null head hediff def", AgeHediff?.defName ?? "null age hediff def");

                            for (int A = 0; A < unDisplay.Count; A++)
                            {
                                for (int B = 0; B < input.apparel.WornApparelCount; B++)
                                {
                                    for (int C = 0; C < unDisplay[A].apparel.Count; C++)
                                    {
                                        if (input.apparel.WornApparel[B].def == unDisplay[A].apparel[C])
                                        {
                                            if (!unDisplay[A].displayHair)
                                            {
                                                HealHediff(input, HairHediff);
                                            }
                                            if (!unDisplay[A].displayHead)
                                            {
                                                HealHediff(input, HeadHediff);
                                            }
                                            if (!unDisplay[A].displayEar)
                                            {
                                                HealHediff(input, AgeHediff);
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        input.Drawer.renderer.graphics.ResolveAllGraphics();
                        break;
                    }
                }

            }
        }

        private void FinalHediffLogic(Pawn input, List<Hediff> hediffs, string HediffPrefixBeforeAgeElement, string AgeElement, HediffDef MustAddHediff)
        {
            bool nullHediff = true;
            for (int i = 0; i < hediffs.Count; i++)
            {
                Hediff hediff = hediffs[i];
                if (hediff.def.defName.Contains(HediffPrefixBeforeAgeElement) && hediff.def != MustAddHediff)
                {
                    HealHediff(input, hediff.def);
                    AddHediff(input, MustAddHediff);
                    nullHediff = false;
                    //break;
                }
            }
            if (nullHediff)
            {
                AddHediff(input, MustAddHediff);
            }
        }

        public static void HealHediff(Pawn input, HediffDef hedifftoheal)
        {
            if (input.health.hediffSet.HasHediff(hedifftoheal, false))
            {
                input.health.hediffSet.GetFirstHediffOfDef(hedifftoheal, false).Heal(10f);
            }
        }

        //특정 헤디프 보유 여부 판단 후 추가
        public static void AddHediff(Pawn input, HediffDef hedifftoadd)
        {
            if (!input.health.hediffSet.HasHediff(hedifftoadd, false))
            {
                input.health.AddHediff(hedifftoadd);
            }
        }

        private void FinalHairHeadNumLogic(List<Hediff> hediffs, List<ElementNumber> elementNumbers, string finalPropertyName, out string finalRandNum)
        {
            finalRandNum = "_00";
            for (int i = 0; i < elementNumbers.Count; i++)
            {
                if (finalPropertyName == elementNumbers[i].elementName)
                {
                    Random rand = new Random();
                    int randnum;
                    string elementPostFix = "";
                    if (UseElementPostFix(hediffs, elementNumbers[i].elementPostFix, out int maxNumber, out elementPostFix))
                    {
                        randnum = rand.Next(0, maxNumber);
                    }
                    else
                    {
                        randnum = rand.Next(0, elementNumbers[i].maxNumber);
                    }
                    finalRandNum = randnum < 10 ? $"{elementPostFix}_0{randnum}" : $"{elementPostFix}_{randnum}";
                    break;
                }
            }
        }

        private static bool UseElementPostFix(List<Hediff> hediffs, List<ElementPostFix> elementPostFix, out int maxNumber, out string elementPostFixName)
        {
            elementPostFixName = "";
            maxNumber = 0;
            for (int j = 0; j < elementPostFix.Count; j++)
            {
                for (int k = 0; k < elementPostFix[j].hediffDefName.Count; k++)
                {
                    for (int l = 0; l < hediffs.Count; l++)
                    {
                        if (hediffs[l].def == elementPostFix[j].hediffDefName[k])
                        {
                            elementPostFixName = elementPostFix[j].elementPostFixName;
                            maxNumber = elementPostFix[j].maxNumber;
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        private string FinalPropertyNameLogic(UnUsed unUsedType, string elementPropertyName, out string finalPropertyName)
        {
            finalPropertyName = elementPropertyName ?? "";
            if ((bool)(unUsedType?.unUsedList?.Any()))
            {
                foreach (string v in unUsedType.unUsedList)
                {
                    if (v == elementPropertyName)
                    {
                        finalPropertyName = unUsedType.elseUse;
                        break;
                    }
                }
            }
            return finalPropertyName;
        }
        private void PropertyLogic(List<Hediff> hediffs, List<PropertyName> propertyType, out string propertyName)
        {
            propertyName = "";
            bool noMatchProperty = true;

            if (propertyType != null)
            {
                foreach (Hediff hediff in hediffs)
                {
                    for (int i = 0; i < propertyType.Count; i++)
                    {
                        for (int j = 0; j < propertyType[i].hediffDefName.Count; j++)
                        {
                            if (hediff.def == propertyType[i].hediffDefName[j])
                            {
                                propertyName = propertyType[i].propertyName;
                                noMatchProperty = false;
                                break;
                            }
                        }
                        if (!noMatchProperty)
                        {
                            break;
                        }
                    }
                    if (!noMatchProperty)
                    {
                        break;
                    }
                }
            }
            else
            {
                Log.Error($"{propertyType}is null List");
            }
        }
        private void TestErrorLog(params string[] arr1)
        {
            if (ChangeLook_Setting.ChangeLookTestLog)
            {
                for (int i = 0; i < arr1.Length; i++)
                {
                    Log.Error(arr1[i]);
                }
            }
        }
        private void DefaultHediffLogic(Pawn input, List<Hediff> hediffs, DefaultPropertyHediff defaultPropertyHediff)
        {
            defaultPropertyHediff.ifPawnDoesntHaveTheseHeadifs.Add(defaultPropertyHediff.hediffDefName);
            bool NeedAddHediff = true;

            for (int i = 0; i < hediffs.Count; i++)
            {
                for (int j = 0; j < defaultPropertyHediff.ifPawnDoesntHaveTheseHeadifs.Count; j++)
                {
                    if (hediffs[i].def == defaultPropertyHediff.ifPawnDoesntHaveTheseHeadifs[j])
                    {
                        TestErrorLog($"Pawn already has this Hediff : {hediffs[i].def.defName}");
                        NeedAddHediff = false;
                        break;
                    }
                }
                if (!NeedAddHediff)
                {
                    break;
                }
            }
            if (NeedAddHediff)
            {
                AddHediff(input, defaultPropertyHediff.hediffDefName);
                TestErrorLog($"Add default Hediff : {defaultPropertyHediff.hediffDefName.defName}");
            }
        }
    }

}
