﻿using AlienRace;
using HarmonyLib;
using RimWorld;
using System;
using Verse;
using Verse.AI;

namespace ChangeLook
{
    [StaticConstructorOnStartup]
    public static class ChangeLook_Harmony
    {
        static ChangeLook_Harmony()
        {
            Harmony harmonyInstance = new Harmony("ChangeLook_Harmony");
            harmonyInstance.Patch(AccessTools.Method(typeof(Pawn_AgeTracker), "BirthdayBiological", null, null), null, new HarmonyMethod(patchType, "BirthdayBiologicalPostFix", null), null, null);
            harmonyInstance.Patch(AccessTools.Method(typeof(Pawn_AgeTracker), "RecalculateLifeStageIndex", null, null), null, new HarmonyMethod(patchType, "RecalculateLifeStageIndexPostfix", null), null, null);
            harmonyInstance.Patch(AccessTools.Method(typeof(JobDriver), "Cleanup", null, null), null, new HarmonyMethod(patchType, "CleanupPostfix", null), null, null);
            harmonyInstance.Patch(AccessTools.Method(typeof(Pawn_ApparelTracker), "Wear", null, null), null, new HarmonyMethod(patchType, "WearPostfix", null), null, null);
            harmonyInstance.Patch(AccessTools.Method(typeof(Pawn_ApparelTracker), "TryDrop", new Type[]
                {
                typeof(Apparel),
                typeof(Apparel).MakeByRefType(),
                typeof(IntVec3),
                typeof(bool)
                }, null), null, new HarmonyMethod(patchType, "TryDropPostfix", null), null, null);
            Log.Message("ChangeLook_patched");
        }

        [HarmonyPostfix]
        public static void TryDropPostfix(Pawn ___pawn, Apparel ap)
        {
            foreach (ChangeLookDef changeLookDef in DefDatabase<ChangeLookDef>.AllDefs)
            {
                if (changeLookDef.targetRace == ___pawn.def)
                {
                    for (int i = 0; i < changeLookDef.unDisplay.Count; i++)
                    {
                        for (int j = 0; j < changeLookDef.unDisplay[i].apparel.Count; j++)
                        {
                            if (ap.def == changeLookDef.unDisplay[i].apparel[j])
                            {
                                changeLookDef.StartChangeLook(___pawn, "TryDropPostfix");
                            }
                        }

                    }
                }
            }
        }

        [HarmonyPostfix]
        public static void WearPostfix(Pawn ___pawn, Apparel newApparel)
        {
            foreach (ChangeLookDef changeLookDef in DefDatabase<ChangeLookDef>.AllDefs)
            {
                if (changeLookDef.targetRace == ___pawn.def)
                {
                    for (int i = 0; i < changeLookDef.unDisplay.Count; i++)
                    {
                        for (int j = 0; j < changeLookDef.unDisplay[i].apparel.Count; j++)
                        {
                            if (newApparel.def == changeLookDef.unDisplay[i].apparel[j])
                            {
                                changeLookDef.StartChangeLook(___pawn, "WearPostfix");
                            }
                        }
                    }

                }
            }
        }

        [HarmonyPostfix]
        public static void BirthdayBiologicalPostFix(Pawn ___pawn)
        {
            foreach (ChangeLookDef changeLookDef in DefDatabase<ChangeLookDef>.AllDefs)
            {
                if (changeLookDef.targetRace == ___pawn.def)
                {
                    changeLookDef.StartChangeLook(___pawn, "BirthdayBiologicalPostFix");
                }
            }
        }

        [HarmonyPostfix]
        public static void RecalculateLifeStageIndexPostfix(Pawn_AgeTracker __instance)
        {
            Pawn value;

            if (Current.ProgramState == ProgramState.Playing && (value = Traverse.Create(__instance).Field("pawn").GetValue<Pawn>()).def is ThingDef_AlienRace && value.Drawer.renderer.graphics.AllResolved)
            {
                foreach (ChangeLookDef changeLookDef in DefDatabase<ChangeLookDef>.AllDefs)
                {
                    if (changeLookDef.targetRace == value.def)
                    {
                        ThingComp_ChangeLook thingComp_Change = value.TryGetComp<ThingComp_ChangeLook>();
                        if (thingComp_Change.chachedAgeBiologicalYears != __instance.AgeBiologicalYears)
                        {
                            changeLookDef.StartChangeLook(value, "RecalculateLifeStageIndexPostfix");
                            thingComp_Change.chachedAgeBiologicalYears = __instance.AgeBiologicalYears;
                        }
                    }
                }
            }
        }

        [HarmonyPostfix]
        public static void CleanupPostfix(JobDriver __instance)
        {
            if (__instance is JobDriver_UseStylingStation)
            {
                Pawn value;

                if (Current.ProgramState == ProgramState.Playing && (value = Traverse.Create(__instance).Field("pawn").GetValue<Pawn>()).def is ThingDef_AlienRace && value.Drawer.renderer.graphics.AllResolved)
                {
                    foreach (ChangeLookDef changeLookDef in DefDatabase<ChangeLookDef>.AllDefs)
                    {
                        if (changeLookDef.targetRace == value.def)
                        {
                            changeLookDef.StartChangeLook(value, "CleanupPostfix");
                        }
                    }
                }
            }
        }

        private static readonly Type patchType = typeof(ChangeLook_Harmony);
    }
}
