﻿using RimWorld;
using System;
using System.Text;
using Verse;

namespace Mincho_ExpandablePlants
{
    public class Building_PlantGrower_Minchoshroom : Building_PlantGrower, IPlantToGrowSettable
    {
        public override string GetInspectString()
        {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.Append(base.GetInspectString());
            string text = InspectStringPartsFromComps();
            if (!text.NullOrEmpty())
            {
                if (stringBuilder.Length > 0)
                {
                    stringBuilder.AppendLine();
                }

                stringBuilder.Append(text);
            }
            if (Spawned)
            {
                if (!text.NullOrEmpty())
                { text += "\n"; }
                if (GrowthSeasonNow(Position, Map, true))
                {
                    text += "GrowSeasonHereNow".Translate();
                }
                else
                {
                    text += "CannotGrowBadSeasonTemperature".Translate();
                }
            }
            return text;
        }

        public static bool GrowthSeasonNow(IntVec3 c, Map map, bool forSowing = false)
        {
            Room roomOrAdjacent = c.GetRoomOrAdjacent(map, RegionType.Set_All);
            if (roomOrAdjacent == null)
            {
                return false;
            }
            float temperature = c.GetTemperature(map);
            return temperature <= 0f;
        }

        public new bool CanAcceptSowNow()
        {
            return base.CanAcceptSowNow() && GrowthSeasonNow(Position, Map, true);
        }
    }

    public class Building_PlantGrower_Minchoshroom_SeedPlease : Building_PlantGrower
    {
        public override string GetInspectString()
        {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.Append(base.GetInspectString());
            string text = InspectStringPartsFromComps();
            if (!text.NullOrEmpty())
            {
                if (stringBuilder.Length > 0)
                {
                    stringBuilder.AppendLine();
                }

                stringBuilder.Append(text);
            }
            if (Spawned)
            {
                if (!text.NullOrEmpty())
                { text += "\n"; }
                if (PlantUtility.GrowthSeasonNow(base.Position, base.Map, true))
                {
                    text += "GrowSeasonHereNow".Translate();
                }
                else
                {
                    text += "CannotGrowBadSeasonTemperature".Translate();
                }
            }
            return text;
        }
    }
}
