﻿using RimWorld;
using Verse;


namespace Mincho_The_Mint_Choco_Slime
{
    internal class CompProperties_SecondLayer : CompProperties
    {
        public GraphicData graphicData = null;
        public bool drawOnlySouth = false;

        public AltitudeLayer altitudeLayer = AltitudeLayer.MoteOverhead;

        public float Altitude
        {
            get
            {
                return altitudeLayer.AltitudeFor();
            }
        }

        public CompProperties_SecondLayer()
        {
            compClass = typeof(CompSecondLayer);
        }
    }
    internal class CompSecondLayer : ThingComp
    {
        bool drawOnlySouth = false;
        private Graphic graphicInt;
        public override void PostDeSpawn(Map map)
        {
            base.PostDeSpawn(map);
            drawOnlySouth = Props.drawOnlySouth;
        }

        public CompProperties_SecondLayer Props
        {
            get
            {
                return (CompProperties_SecondLayer)props;
            }
        }

        public virtual Graphic Graphic
        {
            get
            {
                if (graphicInt == null)
                {
                    if (Props.graphicData == null)
                    {
                        Log.ErrorOnce(parent.def + "MinchoFurniture - has no SecondLayer graphicData but we are trying to access it.", 764532);
                        return BaseContent.BadGraphic;
                    }
                    graphicInt = Props.graphicData.Graphic;
                }
                return graphicInt;
            }
        }

        public override void PostDraw()
        {
            if (drawOnlySouth && parent.Rotation == Rot4.South)
            {
                Graphic.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
            }
            else
            {
                Graphic.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
            }
        }
    }

    internal class CompProperties_SecondLayerFollow : CompProperties
    {
        public GraphicData graphicData = null;
        public bool drawOnlySouth = false;

        public AltitudeLayer altitudeLayer = AltitudeLayer.MoteOverhead;

        public float Altitude
        {
            get
            {
                return altitudeLayer.AltitudeFor();
            }
        }

        public CompProperties_SecondLayerFollow()
        {
            compClass = typeof(CompSecondLayerFollow);
        }
    }
    internal class CompSecondLayerFollow : ThingComp
    {
        bool drawOnlySouth = false;
        private Graphic graphicInt;
        public override void PostDeSpawn(Map map)
        {
            base.PostDeSpawn(map);
            drawOnlySouth = Props.drawOnlySouth;
        }

        public CompProperties_SecondLayerFollow Props
        {
            get
            {
                return (CompProperties_SecondLayerFollow)props;
            }
        }

        public virtual Graphic Graphic
        {
            get
            {
                if (graphicInt == null)
                {
                    if (Props.graphicData == null)
                    {
                        Log.ErrorOnce(parent.def + "MinchoFurniture - has no SecondLayer graphicData but we are trying to access it.", 764532);
                        return BaseContent.BadGraphic;
                    }
                    graphicInt = Props.graphicData.GraphicColoredFor(parent);
                }
                return graphicInt;
            }
        }

        public override void PostDraw()
        {
            if (drawOnlySouth && parent.Rotation == Rot4.South)
            {
                Graphic.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
            }
            else
            {
                Graphic.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
            }
        }
    }



    internal class CompProperties_SecondLayerOnOffable : CompProperties
    {
        public GraphicData graphicDataOn = null;
        public GraphicData graphicDataOff = null;

        public AltitudeLayer altitudeLayer = AltitudeLayer.MoteOverhead;

        public float Altitude
        {
            get
            {
                return altitudeLayer.AltitudeFor();
            }
        }

        public CompProperties_SecondLayerOnOffable()
        {
            compClass = typeof(CompSecondLayerOnOffable);
        }
    }
    internal class CompSecondLayerOnOffable : ThingComp
    {
        public CompRefuelable refuelableComp;
        public CompFlickable compFlickable;
        public Graphic graphicIntOn;
        public Graphic graphicIntOff;
        public override void PostSpawnSetup(bool respawningAfterLoad)
        {
            base.PostSpawnSetup(respawningAfterLoad);
            refuelableComp = parent.GetComp<CompRefuelable>();
            compFlickable = parent.GetComp<CompFlickable>();
        }
        public CompProperties_SecondLayerOnOffable Props
        {
            get
            {
                return (CompProperties_SecondLayerOnOffable)props;
            }
        }

        public virtual Graphic GraphicOn
        {
            get
            {
                if (graphicIntOn == null)
                {
                    if (Props.graphicDataOn == null)
                    {
                        Log.ErrorOnce(parent.def + "MinchoFurniture - has no SecondLayerOnOffable graphicData but we are trying to access it.", 764532);
                        return BaseContent.BadGraphic;
                    }
                    graphicIntOn = Props.graphicDataOn.Graphic;
                }
                return graphicIntOn;
            }
        }
        public virtual Graphic GraphicOff
        {
            get
            {
                if (graphicIntOff == null)
                {
                    if (Props.graphicDataOff == null)
                    {
                        Log.ErrorOnce(parent.def + "MinchoFurniture - has no SecondLayerOnOffable graphicData but we are trying to access it.", 764532);
                        return BaseContent.BadGraphic;
                    }
                    graphicIntOff = Props.graphicDataOff.Graphic;
                }
                return graphicIntOff;
            }
        }

        public override void PostDraw()
        {
            //스위치 온 그리고 연료 유
            if (compFlickable.SwitchIsOn && refuelableComp.HasFuel)
            {
                DrawCallOn();
            }
            else
            {
                //Log.Message("off");
                DrawCallOff();
            }
        }
        public void DrawCallOn()
        {
            GraphicOn.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
        }
        public void DrawCallOff()
        {
            GraphicOff.Draw(GenThing.TrueCenter(parent.Position, parent.Rotation, parent.def.size, Props.Altitude), parent.Rotation, parent, 0f);
        }
    }

}
