﻿using HarmonyLib;
using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using Verse;
using static WearSetting.WearSettingMethod;

namespace WearSetting
{
    [StaticConstructorOnStartup]
    public static class Harmony_EquipmentUtility
    {
        static Harmony_EquipmentUtility()
        {
            var harmony = new Harmony("Mincho_The_Mint_Choco_Slime");

            harmony.Patch(AccessTools.Method(typeof(EquipmentUtility), "CanEquip", new Type[]
                {
                typeof(Thing),
                typeof(Pawn),
                typeof(string).MakeByRefType(),
                typeof(bool)
                }, null), null, new HarmonyMethod(patchType, "CanEquipPostfix", null), null, null);
        }

        public static void CanEquipPostfix(ref bool __result, Thing thing, Pawn pawn, ref string cantReason)
        {
            if (__result)
            {
                ThingDef thingDef = thing.def;
                bool isApparel = thingDef.IsApparel;
                bool isWeapon = thingDef.IsWeapon;
                IEnumerable<ThingComp_WearSetting> pawn_ThingComp_WearSetting = pawn.GetComps<ThingComp_WearSetting>();
                bool test = false;
                TestError(test,"1");
                //무기 잠금 상태에서 무기 못 바꾸게
                if (isWeapon)
                {
                    foreach (var (wearSetting, wearList) in
                            from thingComp_WearSetting in pawn_ThingComp_WearSetting
                            from wearSetting in thingComp_WearSetting.Props.wearSetting
                            from wearList in wearSetting.wearList
                            select (wearSetting, wearList))
                    {
                        foreach (ThingWithComps equipment in pawn.equipment.AllEquipmentListForReading)
                        {
                            if (equipment.def == wearList && wearSetting.lockWeapon && isWeapon && equipment.def.IsWeapon)
                            {
                                __result = false;
                                if (wearSetting.lockWeaponCantReason == null)
                                {
                                    cantReason = string.Format("LockWeaponCantReason_Default".Translate(
                                        pawn.def.LabelCap,
                                        pawn.LabelShort,
                                        thingDef.LabelCap
                                        ));
                                }
                                else
                                {
                                    cantReason = string.Format(wearSetting.lockWeaponCantReason.Translate(
                                        pawn.def.LabelCap,
                                        pawn.LabelShort,
                                        thingDef.LabelCap
                                        ));
                                }
                                return;
                            }

                        }
                    }
                }

                //무기 장비 못 줍게
                foreach (var wearSetting in
                        from thingComp_WearSetting in pawn_ThingComp_WearSetting
                        from wearSetting in thingComp_WearSetting.Props.wearSetting
                        from wearList in wearSetting.wearList
                        where (isApparel || isWeapon) && thingDef == wearList
                        select wearSetting)
                {

                    TestError(test, "스타트");
                    TestError(test, wearSetting.label);
                    WearSettingBoolMaker(wearSetting,
                                         pawn,
                                         out WearSettingBool wearSettingBool,
                                         out WearSettingBoolString wearSettingBoolString);


                    TestError(test, "wearSettingBool : " + wearSettingBool.ToString());
                    if (wearSettingBool.canWearBool)
                    {
                        __result = true;
                        return;
                    }

                    if (wearSettingBool.cantWearBool)
                    {
                        CantBoolStringMaker(out __result, thing, pawn, out cantReason, wearSetting, wearSettingBool, wearSettingBoolString);
                        TestError(test, "2");
                        return;
                    }
                }


                TestError(test, "3");
                //OnlyWearListApparelLogic
                if (isApparel)
                {
                    TestError(test, "4");
                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.OnSkin))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.onSkin == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }

                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.Shell))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.shell == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }

                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.Middle))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.middle == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }

                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.Belt))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.belt == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }

                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.Overhead))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.overhead == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }

                    if (thingDef.apparel.layers.Contains(ApparelLayerDefOf.EyeCover))
                    {
                        foreach (var wearSetting in
                                from thingComp_WearSetting in pawn_ThingComp_WearSetting
                                from wearSetting in thingComp_WearSetting.Props.wearSetting
                                where wearSetting.onlyWearListApparel.eyeCover == true
                                where !wearSetting.wearList.Contains(thingDef)
                                from wearList in wearSetting.wearList
                                select wearSetting)
                        {
                            if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                            {
                                return;
                            }
                        }
                    }
                }

                //무기만
                if (isWeapon)
                {
                    foreach (var wearSetting in
                            from thingComp_WearSetting in pawn_ThingComp_WearSetting
                            from wearSetting in thingComp_WearSetting.Props.wearSetting
                            where wearSetting.onlyWearListApparel.weapon == true
                            where !wearSetting.wearList.Contains(thingDef)
                            from wearList in wearSetting.wearList
                            select wearSetting)
                    {
                        if (OnlyWearListApparelLogic(ref __result, thing, pawn, ref cantReason, wearSetting))
                        {
                            return;
                        }
                    }
                }

            }

        }

        private static void TestError(bool test,string error)
        {
            if (test)
            {
                Log.Error(error);
            }
        }

        private static bool OnlyWearListApparelLogic(ref bool __result, Thing thing, Pawn pawn, ref string cantReason, WearSetting wearSetting)
        {
            WearSettingBoolMaker(wearSetting,
                                 pawn,
                                 out WearSettingBool wearSettingBool,
                                 out WearSettingBoolString wearSettingBoolString);
            if (wearSettingBool.canWearBool)
            {
                CantBoolStringMaker(out __result, thing, pawn, out cantReason, wearSetting, wearSettingBool, wearSettingBoolString);
                return true;
            }
            return false;
        }

        private static void CantBoolStringMaker(out bool __result, Thing thing, Pawn pawn, out string cantReason, WearSetting wearSetting, WearSettingBool wearSettingBool, WearSettingBoolString wearSettingBoolString)
        {
            __result = false;

            BoolString boolString = wearSetting.boolString;

            BoolString(boolString.ageFalse.Translate(), boolString.ageTrue.Translate(), wearSettingBool.age, out string ageBoolString);
            BoolString(boolString.genderFalse.Translate(), boolString.genderTrue.Translate(), wearSettingBool.gender, out string genderBoolString);
            BoolString(boolString.bodyTypeFalse.Translate(), boolString.bodyTypeTrue.Translate(), wearSettingBool.bodyType, out string bodyTypeBoolString);
            BoolString(boolString.orHediffFalse.Translate(), boolString.orHediffTrue.Translate(), wearSettingBool.orHediff, out string orHediffBoolString);
            BoolString(boolString.andHediffFalse.Translate(), boolString.andHediffTrue.Translate(), wearSettingBool.andHediff, out string andHediffBoolString);
            BoolString(boolString.forbiddenHediffFalse.Translate(), boolString.forbiddenHediffTrue.Translate(), !wearSettingBool.forbiddenHediff, out string forbiddenHediffBoolString);
            BoolString(boolString.mustWearWithFalse.Translate(), boolString.mustWearWithTrue.Translate(), wearSettingBool.mustWearWith, out string mustWearWithBoolString);

            if (wearSetting.cantReason == null)
            {
                cantReason = CantReasonMaker("WearSettingForbidden", thing, pawn, wearSetting, wearSettingBoolString, ageBoolString, genderBoolString, bodyTypeBoolString, orHediffBoolString, andHediffBoolString, forbiddenHediffBoolString, mustWearWithBoolString);
            }
            else
            {
                cantReason = CantReasonMaker(wearSetting.cantReason, thing, pawn, wearSetting, wearSettingBoolString, ageBoolString, genderBoolString, bodyTypeBoolString, orHediffBoolString, andHediffBoolString, forbiddenHediffBoolString, mustWearWithBoolString);
            }
        }

        private static string CantReasonMaker(string label, Thing thing, Pawn pawn, WearSetting wearSetting, WearSettingBoolString wearSettingBoolString, string ageBoolString, string genderBoolString, string bodyTypeBoolString, string orHediffBoolString, string andHediffBoolString, string forbiddenHediffBoolString, string mustWearWithBoolString)
        {
            return string.Format(label.Translate(pawn.def.LabelCap,//0
                                                 pawn.LabelShort,//1
                                                 thing.def.LabelCap,//2
                                                 ageBoolString,//3
                                                 wearSetting.condition.age.min,//4
                                                 wearSetting.condition.age.max,//5
                                                 genderBoolString,//6
                                                 wearSettingBoolString.genderString,//7
                                                 bodyTypeBoolString,//8
                                                 wearSettingBoolString.bodyTypeString,//9
                                                 orHediffBoolString,//10
                                                 wearSettingBoolString.orHediffString,//11
                                                 andHediffBoolString,//12
                                                 wearSettingBoolString.andHediffString,//13
                                                 forbiddenHediffBoolString,//14
                                                 wearSettingBoolString.forbiddenHediffString,//15
                                                 mustWearWithBoolString,//16
                                                 wearSettingBoolString.mustWearWithString));//17
        }

        private static readonly Type patchType = typeof(Harmony_EquipmentUtility);
    }
}
