﻿using RimWorld;
using System;
using System.Collections.Generic;
using Verse;

namespace WearSetting
{
    [StaticConstructorOnStartup]
    //대상 종족에 콤프 추가
    public class AddComp
    {
        static AddComp()
        {
            foreach (var wearSettingDef in DefDatabase<WearSettingDef>.AllDefs)
            {
                CompProperties_WearSetting compProperties_WearSetting = new CompProperties_WearSetting
                {
                    targetRace = wearSettingDef.targetRace,
                    wearSetting = wearSettingDef.wearSetting,
                };
                wearSettingDef.targetRace.comps.Add(compProperties_WearSetting);
            };
        }
    }

    public class CompProperties_WearSetting : CompProperties
    {
        public CompProperties_WearSetting()
        {
            this.compClass = typeof(ThingComp_WearSetting);
        }
        public ThingDef targetRace = new ThingDef();
        public List<WearSetting> wearSetting = new List<WearSetting>();
    }
    public class ThingComp_WearSetting : ThingComp
    {
        public CompProperties_WearSetting Props
        {
            get
            {
                return (CompProperties_WearSetting)this.props;
            }
        }
        public override void CompTickRare()
        {
            if (pawn != null && pawn.Spawned)
            {
                base.CompTickRare();


                string addHediffString = "";
                string partsToAffectString = "";

                foreach (WearSetting wearSetting in Props.wearSetting)
                {
                    WearSettingMethod.WearSettingBoolMaker(wearSetting,
                                                           pawn,
                                                           out WearSettingBool wearSettingBool,
                                                           out WearSettingBoolString wearSettingBoolString);

                    foreach (ThingDef wearThing in wearSetting.wearList)
                    {
                        //의복
                        if (wearThing.IsApparel)
                        {
                            for (int k = 0; k < pawn.apparel.WornApparel.Count; k++)
                            {
                                Apparel wornApparel = pawn.apparel.WornApparel[k];

                                bool onSkin = wearSetting.onlyWearListApparel.onSkin && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.OnSkin);
                                bool shell = wearSetting.onlyWearListApparel.shell && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.Shell);
                                bool middle = wearSetting.onlyWearListApparel.middle && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.Middle);
                                bool belt = wearSetting.onlyWearListApparel.belt && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.Belt);
                                bool overhead = wearSetting.onlyWearListApparel.overhead && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.Overhead);
                                bool eyeCover = wearSetting.onlyWearListApparel.eyeCover && wornApparel.def.apparel.layers.Contains(ApparelLayerDefOf.EyeCover);

                                bool apparel = onSkin || shell || middle || belt || overhead || eyeCover;



                                if (wornApparel.def == wearThing)
                                {
                                    if (wearSettingBool.canWearBool)
                                    {
                                        AddHediff(ref addHediffString, ref partsToAffectString, wearSetting, wornApparel, wearSettingBoolString);
                                        LockApparel(pawn.apparel, wearSetting, wornApparel, wearSettingBoolString, wearSetting.lockApparelMessage);
                                    }

                                    else if (wearSettingBool.cantWearBool)
                                    {
                                        if (wearSetting.forceDrop || wearSettingBool.forceDropHediffBool)
                                        {
                                            ApparelDrop(wearSetting, wearSettingBoolString, wornApparel);
                                        }
                                    }

                                    MakeWeaponAndApparel(wearSetting, wornApparel);
                                }

                                //컨디션에 부합할 경우 특정 레이어의 옷을 전용것만 입기
                                if (!wearSetting.wearList.Contains(wornApparel.def) && wearSettingBool.canWearBool && wearSetting.forceDropOnlyMatchCondition && apparel)
                                {
                                    ApparelDrop(wearSetting, wearSettingBoolString, wornApparel);
                                }

                                //해당 컨디션에 부합하지 않더라도 다른 컨디션에서 부합할 경우 벗지 않게 하기
                                //if (wornApparel.def != wearThing && wearSettingBool.canWearBool && wearSetting.forceDropOnlyMatchCondition && apparel)
                                //{
                                //    TestError(test, "3");
                                //    ApparelDrop(wearSetting, wearSettingBoolString, wornApparel);
                                //}
                            }

                        }

                        //무기
                        else if (wearThing.IsWeapon)
                        {
                            for (int k = 0; k < pawn.equipment.AllEquipmentListForReading.Count; k++)
                            {
                                ThingWithComps equipment = pawn.equipment.AllEquipmentListForReading[k];
                                if (wearThing == equipment.def)
                                {
                                    if (wearSettingBool.cantWearBool)
                                    {
                                        if (wearSetting.forceDrop || wearSettingBool.forceDropHediffBool)
                                        {
                                            pawn.equipment.TryDropEquipment(equipment, out ThingWithComps thingWithCompsTemp, pawn.Position, true);

                                            if (wearSetting.destroyAfterDrop)
                                            {
                                                thingWithCompsTemp.Destroy();
                                            }

                                            if (wearSetting.forceDropMessage != null)
                                            {
                                                WearSettingMessages(wearSetting.forceDropMessage, wearSetting, equipment, wearSettingBoolString);
                                            }
                                        }
                                    }
                                    else if (wearSettingBool.canWearBool)
                                    {
                                        AddHediff(ref addHediffString, ref partsToAffectString, wearSetting, equipment, wearSettingBoolString);
                                    }

                                    MakeWeaponAndApparel(wearSetting, equipment);
                                }

                                if (wearThing != equipment.def && wearSettingBool.canWearBool && wearSetting.forceDrop && wearSetting.onlyWearListApparel.weapon == true)
                                {
                                    if (wearSettingBool.cantWearBool)
                                    {
                                        if (wearSetting.forceDrop || wearSettingBool.forceDropHediffBool)
                                        {
                                            pawn.equipment.TryDropEquipment(equipment, out ThingWithComps thingWithCompsTemp, pawn.Position, true);

                                            if (wearSetting.destroyAfterDrop)
                                            {
                                                thingWithCompsTemp.Destroy();
                                            }

                                            if (wearSetting.forceDropMessage != null)
                                            {
                                                WearSettingMessages(wearSetting.forceDropMessage, wearSetting, equipment, wearSettingBoolString);
                                            }
                                        }
                                    }
                                    else if (wearSettingBool.canWearBool)
                                    {
                                        AddHediff(ref addHediffString, ref partsToAffectString, wearSetting, equipment, wearSettingBoolString);
                                    }

                                    MakeWeaponAndApparel(wearSetting, equipment);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void ApparelDrop(WearSetting wearSetting, WearSettingBoolString wearSettingBoolString, Apparel wornApparel)
        {
            pawn.apparel.TryDrop(wornApparel, out Apparel apparelTemp, pawn.Position, true);

            if (wearSetting.destroyAfterDrop)
            {
                apparelTemp.Destroy();
            }

            if (wearSetting.forceDropMessage != null)
            {
                WearSettingMessages(wearSetting.forceDropMessage, wearSetting, wornApparel, wearSettingBoolString);
            }
        }


        private void MakeWeaponAndApparel(WearSetting wearSetting, ThingWithComps pawnWorn)
        {
            //무기 장착
            MakeWeapon(wearSetting, pawnWorn);
            //의류 장착
            MakeApparel(wearSetting, pawnWorn);
        }

        private void MakeWeapon(WearSetting wearSetting, ThingWithComps pawnWorn)
        {
            if (wearSetting.makeWeapon != null)
            {
                ThingDef makeWeapon = wearSetting.makeWeapon;
                if (!makeWeapon.IsWeapon)
                {
                    Log.ErrorOnce($"{makeWeapon.defName} is not Weapon", 100);
                }
                else
                {
                    bool noMore = false;
                    for (int m = 0; m < pawn.equipment.AllEquipmentListForReading.Count; m++)
                    {
                        ThingWithComps equipment = pawn.equipment.AllEquipmentListForReading[m];
                        foreach (var dontDrop in wearSetting.makeWeaponDontDrop)
                        {
                            if (equipment?.def == makeWeapon || equipment?.def == dontDrop)
                            {
                                return;
                            }
                        }
                    }
                    if (!noMore)
                    {
                        ThingWithComps weapon = null;
                        if ((bool)pawnWorn?.def.MadeFromStuff && makeWeapon.MadeFromStuff)
                        {
                            foreach (var stuffCategory1 in pawnWorn.Stuff.stuffCategories)
                            {
                                foreach (var stuffCategory2 in makeWeapon.stuffCategories)
                                {
                                    if (stuffCategory1 == stuffCategory2)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, pawnWorn.Stuff);
                                    }
                                }
                            }
                        }
                        else if (makeWeapon.MadeFromStuff)
                        {
                            if (makeWeapon.defaultStuff != null)
                            {
                                weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, makeWeapon.defaultStuff);
                            }
                            else
                            {
                                foreach (var stuffCategory in makeWeapon.stuffCategories)
                                {
                                    if (stuffCategory == StuffCategoryDefOf.Fabric)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, ThingDefOf.Cloth);
                                        break;
                                    }
                                    else if (stuffCategory == StuffCategoryDefOf.Metallic)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, ThingDefOf.Steel);
                                        break;
                                    }
                                    else if (stuffCategory == StuffCategoryDefOf.Woody)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, ThingDefOf.WoodLog);
                                        break;
                                    }
                                    else if (stuffCategory == StuffCategoryDefOf.Leathery)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, ThingDefOf.Leather_Plain);
                                        break;
                                    }
                                    else if (stuffCategory == StuffCategoryDefOf.Stony)
                                    {
                                        weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon, ThingDefOf.Sandstone);
                                        break;
                                    }
                                }
                            }
                        }
                        else
                        {
                            weapon = (ThingWithComps)ThingMaker.MakeThing(makeWeapon);
                        }

                        if (pawnWorn.TryGetQuality(out QualityCategory qualityCategory))
                        {
                            //ArtGenerationContext 외지인 부분도 해줘야 하나?
                            weapon?.TryGetComp<CompQuality>()?.SetQuality(qualityCategory, ArtGenerationContext.Colony);
                        }
                        //장비중 무기 찾기 위해서 필요한 부분임
                        for (int m = 0; m < pawn.equipment.AllEquipmentListForReading.Count; m++)
                        {
                            ThingWithComps equipment = pawn.equipment.AllEquipmentListForReading[m];
                            foreach (var dontDrop in wearSetting.makeWeaponDontDrop)
                            {
                                if (equipment?.def == makeWeapon || equipment?.def == dontDrop)
                                {
                                    return;
                                }
                            }
                            if ((bool)(equipment?.def.IsWeapon))
                            {
                                pawn.equipment.TryDropEquipment(equipment, out ThingWithComps thingWithCompsTemp, pawn.Position, true);
                                foreach (var destroyWeaponPostDrop in wearSetting.destroyWeaponPostDrop)
                                {
                                    if (thingWithCompsTemp.def == destroyWeaponPostDrop)
                                    {
                                        thingWithCompsTemp.Destroy();
                                    }
                                }
                            }

                        }
                        var hitPointsMultiple = Math.Round((double)pawnWorn.HitPoints / (double)pawnWorn.MaxHitPoints, 3);
                        var hitpoint = (int)(weapon.MaxHitPoints * hitPointsMultiple);
                        weapon.HitPoints = hitpoint;
                        pawn.equipment.AddEquipment(weapon);
                    }
                }
            }
        }

        private void MakeApparel(WearSetting wearSetting, ThingWithComps pawnWorn)
        {
            if (!wearSetting.makeApparel.NullOrEmpty())
            {
                foreach (var makeApparel in wearSetting.makeApparel)
                {
                    if (!makeApparel.IsApparel)
                    {
                        Log.ErrorOnce($"{makeApparel.defName} is not Apparel", 100);
                    }
                    else
                    {
                        bool noMore = false;
                        for (int m = 0; m < pawn.apparel.WornApparel.Count; m++)
                        {
                            Apparel wornApparelForMakeApparel = pawn.apparel?.WornApparel[m] ?? null;
                            if (wornApparelForMakeApparel?.def == makeApparel)
                            {
                                return;
                            }
                        }
                        if (!noMore)
                        {
                            ThingWithComps app = null;
                            if ((bool)pawnWorn?.def.MadeFromStuff && makeApparel.MadeFromStuff)
                            {
                                foreach (var stuffCategory1 in pawnWorn.Stuff.stuffCategories)
                                {
                                    foreach (var stuffCategory2 in makeApparel.stuffCategories)
                                    {
                                        if (stuffCategory1 == stuffCategory2)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, pawnWorn.Stuff);
                                        }
                                    }
                                }
                            }
                            else if (makeApparel.MadeFromStuff)
                            {
                                if (makeApparel.defaultStuff != null)
                                {
                                    app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, makeApparel.defaultStuff);
                                }
                                else
                                {
                                    foreach (var stuffCategory in makeApparel.stuffCategories)
                                    {
                                        if (stuffCategory == StuffCategoryDefOf.Fabric)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, ThingDefOf.Cloth);
                                            break;
                                        }
                                        else if (stuffCategory == StuffCategoryDefOf.Metallic)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, ThingDefOf.Steel);
                                            break;
                                        }
                                        else if (stuffCategory == StuffCategoryDefOf.Woody)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, ThingDefOf.WoodLog);
                                            break;
                                        }
                                        else if (stuffCategory == StuffCategoryDefOf.Leathery)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, ThingDefOf.Leather_Plain);
                                            break;
                                        }
                                        else if (stuffCategory == StuffCategoryDefOf.Stony)
                                        {
                                            app = (ThingWithComps)ThingMaker.MakeThing(makeApparel, ThingDefOf.Sandstone);
                                            break;
                                        }
                                    }
                                }
                            }
                            else
                            {
                                app = (ThingWithComps)ThingMaker.MakeThing(makeApparel);
                            }
                            if (pawnWorn.TryGetQuality(out QualityCategory qualityCategory))
                            {
                                //ArtGenerationContext 외지인 부분도 해줘야 하나?
                                app?.TryGetComp<CompQuality>()?.SetQuality(qualityCategory, ArtGenerationContext.Colony);
                            }
                            var hitPointsMultiple = Math.Round((double)pawnWorn.HitPoints / (double)pawnWorn.MaxHitPoints, 3);
                            var hitpoint = (int)(app.MaxHitPoints * hitPointsMultiple);
                            app.HitPoints = hitpoint;
                            pawn.apparel.Wear((Apparel)app);
                            break;
                        }
                    }
                }
            }
        }

        private void LockApparel(Pawn_ApparelTracker apparel, WearSetting wearSetting, Apparel wornApparel, WearSettingBoolString wearSettingBoolString, string label)
        {
            if (wearSetting.lockApparel)
            {
                bool tryLock = true;
                foreach (Apparel lockedApparel in apparel.LockedApparel)
                {
                    if (lockedApparel == wornApparel)
                    {
                        tryLock = false;
                        break;
                    }
                }
                if (tryLock)
                {
                    apparel.Lock(wornApparel);
                    if (wearSetting.lockApparelMessage != null)
                    {
                        WearSettingMessages(label, wearSetting, wornApparel, wearSettingBoolString);
                    }
                }
            }
        }

        private void AddHediff(ref string addHediffString, ref string partsToAffectString, WearSetting wearSetting, Thing thing, WearSettingBoolString wearSettingBoolString)
        {
            HediffDef hediffDef = new HediffDef();
            foreach (AddHediff addHediff in wearSetting.addHediff)
            {
                addHediffString = "";
                partsToAffectString = "";
                for (int k = 0; k < addHediff.hediff.Count; k++)
                {
                    hediffDef = addHediff.hediff[k];
                    if (!pawn.health.hediffSet.HasHediff(hediffDef, false))
                    {
                        if (HediffGiverUtility.TryApply(pawn, hediffDef, addHediff.partsToAffect))
                        {
                            if (k == addHediff.hediff.Count - 1)
                            {
                                addHediffString += hediffDef.LabelCap;
                            }
                            else
                                addHediffString += hediffDef.LabelCap + ", ";
                        }
                    }
                }
                for (int k = 0; k < addHediff.partsToAffect.Count; k++)
                {
                    if (k == addHediff.partsToAffect.Count - 1)
                    {
                        partsToAffectString += addHediff.partsToAffect[k].LabelCap;
                    }
                    else
                        partsToAffectString += addHediff.partsToAffect[k].LabelCap + ", ";
                }
                if (addHediff.addHediffMessage != null && !addHediffString.Equals(""))
                {
                    Messages.Message(addHediff.addHediffMessage.Translate(
                                           pawn.def.LabelCap,
                                           pawn.LabelShort,
                                           thing.def.LabelCap,
                                           //여기 위치 바뀜 주의
                                           addHediffString,
                                           partsToAffectString,
                                           wearSetting.condition.age.min,
                                           wearSetting.condition.age.max,
                                           wearSettingBoolString.genderString,
                                           wearSettingBoolString.bodyTypeString,
                                           wearSettingBoolString.orHediffString,
                                           wearSettingBoolString.andHediffString,
                                           wearSettingBoolString.forbiddenHediffString,
                                           wearSettingBoolString.mustWearWithString
                                           ).CapitalizeFirst(), pawn, MessageTypeDefOf.NeutralEvent, true);
                }
            }
        }



        private void WearSettingMessages(string label, WearSetting wearSetting, Thing thing, WearSettingBoolString wearSettingBoolString)
        {
            Messages.Message(label.Translate(
                                pawn.def.LabelCap,
                                pawn.LabelShort,
                                thing.def.LabelCap,
                                wearSetting.condition.age.min,
                                wearSetting.condition.age.max,
                                wearSettingBoolString.genderString,
                                wearSettingBoolString.bodyTypeString,
                                wearSettingBoolString.orHediffString,
                                wearSettingBoolString.andHediffString,
                                wearSettingBoolString.forbiddenHediffString,
                                wearSettingBoolString.mustWearWithString
                                ).CapitalizeFirst(), pawn, MessageTypeDefOf.NeutralEvent, true);
        }

        public override void PostSpawnSetup(bool respawningAfterLoad)
        {
            base.PostSpawnSetup(respawningAfterLoad);
            this.pawn = (this.parent as Pawn);
        }
        public Pawn pawn { get; private set; }
    }
}
